#include "component_action_on_radius_hone.h"

/* DEPENDENCIES: Goodies, ActEntity */
/* On deletion, give goodies to the ID belonging to ActEntity. */

/* Checks for an entity ot hone in onto. */
static void
_hone_in_check(ComponentActionOnRadiusHone* self, ECS* ecs)
{
	ComponentCircleCollision* circleCollision;

	circleCollision = ecs_component_get(ecs, ECS_COMPONENT_CIRCLE_COLLISION, self->component.id);

	if (circleCollision->collisions.count == 0)
		return;

	/* Find the first object of the specified type, and hone in on it. */
	for (s32 i = 0; i < (s32)circleCollision->collisions.count; i++)
	{
		ComponentGameEntityType* type;
		u32 id;

		id = *(u32*)vector_get(&circleCollision->collisions, i);

		type = ecs_component_get(ecs, ECS_COMPONENT_GAME_ENTITY_TYPE, id);

		if (!type)
			return;

		if (type->type == self->type)
		{
			self->targetID = id;
			self->isHoming = true;
		}
	}
}

/* Hones in on the target. */
static void
_hone_in(ComponentActionOnRadiusHone* self, ECS* ecs)
{
	ComponentPhysics* physics;
	ComponentPhysics* targetPhysics;
	f32 angle;
	f32 speed;

	targetPhysics = ecs_component_get(ecs, ECS_COMPONENT_PHYSICS, self->targetID);
	physics = ecs_component_get(ecs, ECS_COMPONENT_PHYSICS, self->component.id);

	if (!targetPhysics)
		return;

	angle = ATAN
	(
		targetPhysics->position[0], 
		physics->position[0], 
		targetPhysics->position[1],
		physics->position[1]
	);

	speed = self->speed * (self->timer * self->multiplier);

	physics->velocity[0] = -cos(angle) * speed;
	physics->velocity[1] = -sin(angle) * speed;

	physics->angle = angle;

	self->timer++;
}

/* Initializes action on touch give goodies component. */
void
component_action_on_radius_hone_init
(
	ComponentActionOnRadiusHone* self, 
	f32 speed,
	f32 multiplier,
	GameEntityType type
)
{
	self->speed = speed;
	self->multiplier = multiplier;
	self->type = type;
}

/* Ticks aciton on touch give goodies component. */
void
component_action_on_radius_hone_tick(ComponentActionOnRadiusHone* self, ECS* ecs)
{
	if (self->isHoming)
		_hone_in(self, ecs);
	else
		_hone_in_check(self, ecs);
}
