#include "component_fatness.h"

/* Sets fatness level. */
static void
_fatness_set(ComponentFatness* self, ECS* ecs)
{
	ComponentAnimationIdle* animationIdle;
	ComponentHeart* heart;

	animationIdle = ecs_component_get(ecs, ECS_COMPONENT_ANIMATION_IDLE, self->component.id);
	heart = ecs_component_get(ecs, ECS_COMPONENT_HEART, self->heartID);

	animationIdle->baseIndex = (self->value * 2);

	heart->multiplierBonus = self->value;
}

/* Initializes fatness component. */
void
component_fatness_init(ComponentFatness* self, u32* thresholds, u32 thresholdCount, u32 heartID)
{
	self->thresholds = thresholds;
	self->thresholdCount = thresholdCount;
	self->heartID = heartID;
}

/* Ticks fatness component. */
void
component_fatness_tick(ComponentFatness* self, ECS* ecs)
{
	ComponentGoodies* goodies;

	goodies = ecs_component_get(ecs, ECS_COMPONENT_GOODIES, self->component.id);

	self->previousValue = self->value;

	for (s32 i = self->thresholdCount; i > 0; i--)
	{
		u32 value;

		value = self->thresholds[i];

		if (goodies->value >= value)
		{
			self->value = i + 1;
			
			_fatness_set(self, ecs);

			if (self->value > self->previousValue)
				sound_play(&ecs->game->resources.sounds[SOUND_GROW], SOUND_SPECIAL);
			return;
		}

	}


}
