#include "game_over.h"

static void _game_over_entity_init(GameOver* self);

/* Initializes entities for the gameover state. */
static void
_game_over_entity_init(GameOver* self)
{
	self->backgroundID = ecs_entity_add(&self->state->game->ecs);
	self->textID = ecs_entity_add(&self->state->game->ecs);
	self->imageID = ecs_entity_add(&self->state->game->ecs);
	self->descriptionID = ecs_entity_add(&self->state->game->ecs);
	self->menuButtonID = ecs_entity_add(&self->state->game->ecs);

	entity_sprite_init
	(
		&self->state->game->ecs,
		self->backgroundID,
		self->state->game->resources.textures[TEXTURE_STATE_FADE],
		(f32*)GAME_OVER_BACKGROUND_SIZE,
		(f32*)GAME_OVER_BACKGROUND_POSITION
	);

	entity_text_init
	(
		&self->state->game->ecs,
		self->textID,
		&self->state->game->resources.fonts[FONT_BIG],
		STRING_GAME_OVER_TITLE_TEXT,
		strlen(STRING_GAME_OVER_TITLE_TEXT),
		-1,
		(f32*)GAME_OVER_TITLE_TEXT_POSITION,
		(f32*)COLOR_OPAQUE
	);

	entity_sprite_init
	(
		&self->state->game->ecs,
		self->textID,
		self->state->game->resources.textures[TEXTURE_GAME_OVER],
		(f32*)GAME_OVER_IMAGE_SIZE,
		(f32*)GAME_OVER_IMAGE_POSITION
	);
	
	entity_text_init
	(
		&self->state->game->ecs,
		self->descriptionID,
		&self->state->game->resources.fonts[FONT_SMALL],
		STRING_GAME_OVER_TEXT,
		strlen(STRING_GAME_OVER_TEXT),
		-1,
		(f32*)GAME_OVER_TEXT_POSITION,
		(f32*)COLOR_OPAQUE
	);

	entity_menu_button_init
	(
		&self->state->game->ecs,
		self->menuButtonID,
		(f32*)GAME_OVER_MENU_BUTTON_POSITION,
		MENU_BUTTON_RETRY
	);
}

/* Initializes gameover state. */
void
game_over_init(GameOver* self, State* state)
{
	memset(self, '\0', sizeof(GameOver));

	self->state = state;
	
	_game_over_entity_init(self);

	music_play(&self->state->game->resources.music[MUSIC_GAME_OVER], true);
}

/* Ticks gameover state. */
void
game_over_tick(GameOver* self)
{
	ComponentButton* button;

	button = ecs_component_get(&self->state->game->ecs, ECS_COMPONENT_BUTTON, self->menuButtonID);

	if (button->state == BUTTON_PRESSED)
	{
		sound_play(&self->state->game->resources.sounds[SOUND_SELECT], SOUND_PRIORITY);
		self->isChange = true;
	}
}
